	var express    = require('express');
	var bodyParser = require('body-parser');

	var exec       = require('child_process').exec;
	var execSync   = require('child_process').execSync;
	var fs         = require('fs');
	var path       = require('path');
	var fsReaddir = require('fs-readdir');
	var Zip        = require('node-zip');
	var JSZip = require('jszip');

	var exphbs = require('express-handlebars');

	//initializer
	var nanit = require('nanit');
	//load names of available download files
	const downloadFiles = require('./downloadFiles.json');
	const downloadNames = Object.keys(downloadFiles);

	let app = express();

	app.use(bodyParser.urlencoded({
		extended: true
	}));
	app.use(bodyParser.json());

	//set handlebars as view engine
	app.engine('.hbs', exphbs({extname: '.hbs'}));
	app.set('view engine', '.hbs');

	//initialize the git repos
	//TODO error handling is not working properly, error returned is not in let error, but in req + req and res are undefined!
	//TODO add spinner for telling user to wait until initialization is done
	nanit.initialize(downloadFiles, function(req,res,err){
	// console.log('err in nanit.initialize:', err);
	// console.log('req in nanit.initialize:', req);
	// console.log('res in nanit.initialize:', res);

	let response = 
	          'There was an error during cloning the git repositories. ' +
	          'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
	          'and add the following information: ' + req;
		if(err){
			console.log('nanit.initialize error');
			res.send(response);
			return;
		};

	});

	//routes
	app.get('/', function(req, res){
		res.render('home', {downloadFiles: downloadNames});
	});


	app.post('/', function(req, res, next){
		const checkedFiles = Object.keys(req.body);

		if(typeof checkedFiles !== 'undefined' && checkedFiles.length > 0){
			res.render('download', {checkedFiles: checkedFiles});
		}
		else {
			res.render('home', {downloadFiles: downloadNames});
		}
	});


	app.get('/download', function(req, res){
		const promiseArray = []
		const checkedFiles = req.query.checkedFiles.split(',');

	// create a file to stream archive data to. 
	const zip = new JSZip();
	zip.file("README.txt", "text for README");


	backendChecked = checkedFiles.find(function(file){
		return file === 'explorviz-ui-backend';
	});

	if (backendChecked){

		const fileName = backendChecked;
		const downloadURL = downloadFiles[backendChecked]; 

		const backendPromise = new Promise((resolve, reject)=> {

			exec('git pull ' + ' && cd ' + fileName + ' && mvn compile war:war ', (error, stdout, stderr) => {

				if (error) {
					reject(error);
					return;
				}

				console.log(stdout);
				console.log(stderr);

				resolve({zipFileName:'backend.war', path:'explorviz-ui-backend/target/explorviz-ui-backend-1.0-SNAPSHOT.war'});

			}); 
		})

		promiseArray.push(backendPromise);
			} 

	frontendChecked = checkedFiles.find(function(file){
		return file === 'explorviz-ui-frontend';
	});

	if (frontendChecked){
		const fileName = frontendChecked;
		const downloadURL = downloadFiles[frontendChecked]; 

		function frontendPull(){

			return new Promise((resolve, reject) => {
				console.log('in frontendPull()');
				exec('cd ' +  fileName + ' git pull ', (error, stdout, stderr) => {

					if (error) {
						reject(error)
						return;
					}

					console.log(stdout);
					console.log(stderr);
					resolve();
				});
			})
		} 

		function frontendInstallAddons(){
			console.log('in frontendInstallAddons()');
			return new Promise((resolve, reject) => {

				checkedPlugins = checkedFiles.filter(function(file){
					return /\bexplorviz-frontend-plugin-\w*/.exec(file);
				});

				if(checkedPlugins.length !== 0 ){

					checkedPlugins.forEach((pluginName) => {
						exec('cd ' + 'explorviz-ui-frontend ' + 'ember install ' + downloadFiles[pluginName], (error, stdout, stderr) => {
							
							if (error) {
								reject(error);
								return;
							}

							console.log(stdout);
							console.log(stderr);
							
							resolve();

						});

					})
				} else {
					resolve();
				}

			})
		}  

		function frontendBuild(){

			return new Promise((resolve,reject)=> {
				console.log('in frontendBuild()');

	
	exec('cd ' + fileName +' && ember build --environment=production', (error, stdout, stderr) => {

		if (error) {
			reject(error);
			return;
		}

		console.log(stdout);
		console.log(stderr);

		resolve({zipFileName:'frontend', path:'./explorviz-ui-frontend/dist/'});
	});



})
		}

		function frontendAll() {

			return frontendPull().then(function(){
				return frontendInstallAddons().then(function(){
					return frontendBuild()
				})
			})
		}

		promiseArray.push(frontendAll());
		finalizeZip();

	};



	function finalizeZip(){
		
		return streamFilesToZip().then(function(){
			return createZip()
		}).catch(function(error) {let response = 
			'There was an error during your build. ' +
			'Please contact "explorviz-developers-request@listserv.dfn.de" ' +
			'and add the following information: ' + error;
			res.send(response);
		})

	}

	function streamFilesToZip(){
		console.log('in streamFilesToZip');

		return new Promise((resolve,reject)=> {
			
			Promise.all(promiseArray).then(files => {
				console.log('in Promise.all');
				let finishedArray=[];
				files.forEach(file => {
					if (file.zipFileName === 'backend.war'){
						const backendSteam = fsReaddir(file.path).on('data', function(data){
							const backendReadStream = fs.createReadStream(data);
							zip.file(zipFileName, backendReadStream);
						}).on('error', function(){
							console.log('error:', error);
						}).on('finish', function(){
							finishedArray.push('backend finished');
							if(finishedArray.length === files.length){
						resolve();
					}
						})

						// backendStream = fs.createReadStream(file.path);
						// zip.file(file.zipFileName, backendStream);
						// finishedArray.push('backend done');
					}
					if(file.zipFileName === 'frontend'){
						const stream = fsReaddir(file.path)
						.on('error', function(err) {
							console.log('error:', err);
						}).on('data', function(data){
							for (let i=0; i<data.length; i++) {
								const frontendStream = fs.createReadStream(data[i]);
								zip.file(data[i], frontendStream);
							}

						}).on('finish', function(){
							finishedArray.push('frontend finished');
							if(finishedArray.length === files.length){
						resolve();
					}
						})

					}
				
					

				})

			})


		})
	};

	function createZip(){
		console.log('in createZip');

		return new Promise((resolve,reject) => {
			zip.generateNodeStream({type:'nodebuffer',streamFiles:true}).pipe(fs.createWriteStream('explorviz-builds.zip')).on('finish', function () {
				console.log('explorviz-builds.zip written.');
				res.download('explorviz-builds.zip');
			}); 
			
		})

		
	}





});


	app.listen(3000);
	console.log("Downloader started on port 3000.");